/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999, D.Palomo van Es                                   *
 *                                                                       *
 * Permission to use, copy, modify, distribute and sell this software    *
 * and its documentation for any purpose is hereby granted without fee,  *
 * provided that the above copyright notice appear in all copies and     *
 * that both that copyright notice and this permission notice appear     *
 * in supporting documentation. This software is provided "as is"        *
 * without express or implied warranty.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  clpq is a little program that constantly shows the printerqueue for  *
 *  1 printer and can only be used by root when spoolfiles are unreadable*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                      clpq version 0.1                                 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include<stdlib.h>
#include<time.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <curses.h>
#include <signal.h>
#include <string.h>

#define version "0.1"
#define MAXLENGTHFILE     256
#define MAXLENGTHFILELINE 256
#define STARTLINE           8 
#define CYCLETIME 2
#define MAXFILES 100000
#define MAXENTRIES 1000
#define MAXLCNAME 40
#define delay 100
/* some globals */
bool startup;
char Buffer[MAXENTRIES + 1][256];
int Marked = 0;                // The line to be 'marked'
int n;
char dirname[256];
int rrate;

class lp_obj {
public:
   char *cname; //Name of the c file of this object
   char *H;     //Hostname from which it was invoked
   char *P;     //Print-user who invoked the comm.
   char *J;     //Current Job
   char *L;     //Litteral Theline contains id info from passwd file. (for the banner page)
   char *C;     //Classification, also for the banner page
   char *T;     //String to be used as title for pr1
   char *f;     //Formatted, next to be printed file
   char *l;     //like f, but passes control chars and no page breaks
   char *M;     //Send mail to the specified user when Job is done
   char *U;     //name of file to br removed after completion of pr/ing
   char *N;     //name of the file to be printed, blan==>stdinput
   int  jn;
   lp_obj();
   ~lp_obj();
   void FillFields();
};

lp_obj *files[MAXENTRIES];

int Jobdecode(char n[]){
   int i, j, hulp = 0;
   
   for (j = 0; n[j] != 'A'; j++);
   for (i = j + 1; n[i] != 'A'; i++){
      hulp += n[i] - '0';
      if (n[i+1] != 'A')
         hulp *= 10;
      }
   return hulp;
}


void print (WINDOW *stdscr, int marked) {
   int   i = 0,               // Some counters 
         j = 0, 
         y = 0, 
         x = 0;

   char hulp = 0, 
        hulp2[4];             // Not very clean, used for strcat
   static int firstline  = 0; // We start at line 0
   
   int startline = STARTLINE; // Where do you want to start the text
 
   clear();                // Clear window
   
   getmaxyx(stdscr, y, x); // Get size of the window

   // If the 'marked line' falls out of boundaries
   // scroll down.
   if (marked >= y - startline) { 
      firstline = marked - (y - 1 - startline);
      startline--;         // This is a correction on the startline,
                           // when the y-boundary is reached the startline 
                           // shifts (??? ==> maybe a bug ;-) )
   }
   else 
      firstline = 0;
      
   move (0,0);             // Place your title. 
   attron(COLOR_PAIR(12));
   printw("CLPQ Version %s\n\n", version);
   attroff(COLOR_PAIR(12));
   printw("Press : Up / Down arrow: to move selected up / down
        r : to remove a job
        q : to quit\n\n\n%4s :%-20s %-14s %5s\n","Rank", " Filename", " Owner", "Job");
   
   move (startline, 0);    // Goto startline

   for (i = 0; i < MAXENTRIES + 1; i++)
      for (int j = 0; j < 256; j++)
         Buffer[i][j] = 0;


   for (i = 0; i < MAXENTRIES; i++)
      if (files[i] != NULL) {
         files[i]->jn = Jobdecode(files[i]->cname);
         if (files[i]->N == NULL)
	    sprintf(Buffer[i], "%4d :%-20s %-14s %5d\n", i, " File Unknown", "root", files[i]->jn);
	 else {
         if (!strcmp(files[i]->N, " "))
            strcpy(files[i]->N, " Standard Input");  
	    sprintf(Buffer[i], "%4d :%-20s %-14s %5d\n", i, files[i]->N,
	    files[i]->L, files[i]->jn);
	    }
      }

   for (j = firstline; j < i && j < (y + firstline - startline); j++) {
      if (j == marked)  {
         // attron / attroff : add / remove a attribute from the internal attr. list
         // A_STANDOUT    : Reversed video
         // COLOR_PAIR(n) : Use color pair n
         attron(A_STANDOUT);
         attron(COLOR_PAIR(10));
         // 'printf' - clone to print in the window
         printw("%s", Buffer[j]);
         attroff(A_STANDOUT);
         attroff(COLOR_PAIR(10));
      }
      else 
         printw("%s", Buffer[j]);
   }
}

void lp_obj::FillFields() {
   int i;
   bool several = false;
   char buffer[256];
   ifstream cfile(cname);
   
   for (i = 0;cfile >> buffer && !several; i++) 
      switch(buffer[0]) {
         case 'H': {H = new char[256]; strcpy(H, buffer); H[0] = ' ';}break;
         case 'P': {P = new char[256]; strcpy(P, buffer); P[0] = ' ';}break;
         case 'J': {J = new char[256]; strcpy(J, buffer); J[0] = ' ';}break;
         case 'L': {L = new char[256]; strcpy(L, buffer); L[0] = ' ';}break;
         case 'C': {C = new char[256]; strcpy(C, buffer); C[0] = ' ';}break;
         case 'T': {T = new char[256]; strcpy(T, buffer); T[0] = ' ';}break;
         case 'f': {f = new char[256]; strcpy(f, buffer); f[0] = ' ';}break;
         case 'l': {l = new char[256]; strcpy(l, buffer); l[0] = ' ';}break;
         case 'M': {M = new char[256]; strcpy(M, buffer); M[0] = ' ';}break;
         case 'U': {U = new char[256]; strcpy(U, buffer); U[0] = ' ';}break;
         case 'N': {if (N != NULL) several = true;
                    N = new char[256]; strcpy(N, buffer); N[0] = ' ';}break;
      }
   if (several) 
      strcpy (N, " Several Files");
   cfile.close();
}

lp_obj::lp_obj() {
   cname = NULL; 
   H = NULL;     
   P = NULL;     
   J = NULL;     
   L = NULL;     
   C = NULL;     
   T = NULL;     
   f = NULL;     
   l = NULL;     
   M = NULL;     
   U = NULL;     
   N = NULL;     
}

lp_obj::~lp_obj() {
   delete cname; 
   delete H;
   delete P;
   delete J;
   delete L;
   delete C;
   delete T;
   delete f;
   delete l;
   delete M;
   delete U;
   delete N;
}




void Call_once (WINDOW *stdscr){
   int i;
   char buffer[MAXFILES];
   
   n = 0;
     
   for (i = 0; i < MAXFILES; i++)
      buffer[i] = 0;
       
   ifstream lp_files("/tmp/.temp.___");
   
   if (!lp_files)
      cerr << "Could not write tempfiles propperly" << endl;
   for (i = 0; lp_files >> buffer; i++) {
      files[i] = new lp_obj;
      files[i]->cname = new char[MAXLCNAME];
      strcpy(files[i]->cname, buffer);
   }
   //update the fields:
   for (i = 0; files[i] != NULL; i++) {
      n++;
      files[i]->FillFields();
   }
   print(stdscr, Marked);
   lp_files.close();
} 
  
void init_conf() {
    int i;
    char buffer[256];
    ifstream lp_conf ("conf");
    for (i = 0; lp_conf >> buffer; i++) {
      if (i == 0) 
        strcpy(dirname, buffer);
      if (i == 1)
         rrate = atoi(buffer);
      }
   lp_conf.close();
}
  
void Buffered_ls (WINDOW *stdscr){
   int i;
   ifstream lp_conf ("conf");
   char buffer[256];
   char ls[280] = "ls -A1 ", hulp[280];
   for (i = 0; lp_conf >> buffer; i++)
      if (i == 0) 
        strcpy(hulp , strcat(ls, strcat(buffer,"/c* > /tmp/.temp.___ 2> /tmp/.err.log")));
   lp_conf.close();
   system(hulp);
   if (startup == true)
      Call_once(stdscr);
}

void Kill_all() {
   for (int i = 0; files[i] != NULL; i++){
      delete files[i];
      files[i] = NULL;
   }   
}

void Termination (){
   system("rm /tmp/.temp.___;rm /tmp/.err.log");
}

void Checker(WINDOW *stdscr) {
   int i;
   char buffer[MAXFILES];
   
   Buffered_ls(stdscr);   
   
   ifstream lp_files("/tmp/.temp.___");
   if (!lp_files)
      cerr << "Could not write tempfiles propperly" << endl;
   
   lp_files >> buffer;
   if (files[0] == NULL || strcmp(buffer, files[0]->cname) != 0) {
      if (files[0] != NULL) 
      //   if (strcmp(buffer, files[0]->cname) != 0)//is the firstone finished?
      //      printf("\a");
      if (files[0] == NULL && buffer[0] != '\0') {
            Call_once(stdscr);   
            lp_files.close();
            return;
      }
      else {   
         if (files[0] != NULL) {
            Kill_all();
            Call_once(stdscr);
            lp_files.close();
            return;
         }
      }
   }
   for (i = 1; files[i] != NULL; i++){
      lp_files >> buffer;
         if (files[i] != NULL && strcmp(buffer, files[i]->cname) != 0) {
            Kill_all();
            Call_once(stdscr);
            lp_files.close();
            return;
      }
   }
   lp_files >> buffer;
   if (buffer[0] != '\0')
      Call_once(stdscr);
 lp_files.close();
}

void dequeue(int j) {
   char buff[300];
   sprintf(buff, "rm -f %s/*A%d*A* 2> /tmp/.err.log", dirname, j);
   system (buff);
   ifstream err_ ("/tmp/.err.log");
   int i;
   for (i = 0; err_ >> buff; i++);
   if (i >= 1) {
      move(STARTLINE - 2, 0);
      printw ("Permission Denied!!");
   }
   err_.close();
}

void remove(int job) {
   int i;
   move (STARTLINE - 2,0);
   attron(COLOR_PAIR(11));
   attron(A_STANDOUT);
   if (files[job] != NULL)
      printw("Are you sure you want to remove job %d? [Y]",files[job]->jn);      
   else
      printw("You cannot delete nothing!");
   attroff(A_STANDOUT);
   attroff(COLOR_PAIR(11));
   for (;;) {
      int c = getch();
      if (c == 'y' || c == '\n' || c == 'y') {
        if (files[job] != NULL)
         dequeue(files[job]->jn);   
         break;
      }
      else 
         break;
   }
   
}

static void finish(int sig) {
    endwin();
    Termination();
    exit(0);
}

main(int argc, char *argv[]) {
   (void) signal(SIGINT, finish);  // arrange interrupts to terminate 
   (void) initscr();               // initialize the curses library 
   keypad(stdscr, TRUE);           // enable keyboard mapping
   (void) nonl();                  // tell curses not to do NL->CR/NL on output
   (void) cbreak();                // take input chars one at a time, no wait for \n
   (void) noecho();                // don't echo input
   init_conf();
   
   if (has_colors()) {
      start_color();
      //Simple color assignment, often all we need.
      init_pair(10, COLOR_BLUE, COLOR_WHITE);
      init_pair(11, COLOR_BLUE, COLOR_YELLOW);
      init_pair(12, COLOR_CYAN, COLOR_BLUE);
   }

   halfdelay(rrate);
   startup = true;
   Buffered_ls(stdscr);
   startup = false;
   print(stdscr, 0);                // First Print-out to screen
   for (;;) {                       // refresh, accept single keystroke of input                   
     Checker(stdscr);
     int c = getch();     
      // process the command keystroke 
      switch (c) {
         case KEY_DOWN: (Marked >= n - 1)? Marked = n - 1: Marked++; print(stdscr, Marked);    break;
         case KEY_UP:    (Marked == 0)? Marked = 0: Marked--;
                        print(stdscr,Marked);               break;
         case 'q':      finish(0);                          break;
         case 'r':      remove(Marked);print(stdscr,Marked); (Marked == 0)? Marked = 0: Marked--;break;
      }
   }

    finish(0);                      // we're done 
}
